import { Component, ChangeDetectionStrategy, ElementRef, EventEmitter, Input, NgZone, Output, NgModule } from '@angular/core';
import * as _moment from 'moment';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { take } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

var moment = _moment;
var DlYearModelComponent = /** @class */ (function () {
    function DlYearModelComponent() {
    }
    DlYearModelComponent.getStartOfDecade = function (fromMilliseconds) {
        var startDecade = (Math.trunc(moment(fromMilliseconds).year() / 10) * 10);
        return moment({ year: startDecade }).startOf('year');
    };
    DlYearModelComponent.prototype.onChanges = function (changes) {
    };
    DlYearModelComponent.prototype.getModel = function (milliseconds, selectedMilliseconds) {
        var rowNumbers = [0, 1];
        var columnNumbers = [0, 1, 2, 3, 4];
        var startYear = moment(milliseconds).startOf('year');
        var startDate = DlYearModelComponent.getStartOfDecade(milliseconds);
        var futureYear = startDate.year() + 9;
        var pastYear = startDate.year();
        var activeValue = startYear.valueOf();
        var selectedValue = selectedMilliseconds === null || selectedMilliseconds === undefined
            ? selectedMilliseconds
            : moment(selectedMilliseconds).startOf('year').valueOf();
        var result = {
            viewName: 'year',
            viewLabel: pastYear + "-" + futureYear,
            activeDate: activeValue,
            leftButton: {
                value: moment(startDate).subtract(10, 'years').valueOf(),
                ariaLabel: "Go to " + (pastYear - 10) + "-" + (pastYear - 1),
                classes: {},
            },
            rightButton: {
                value: moment(startDate).add(10, 'years').valueOf(),
                ariaLabel: "Go to " + (futureYear + 1) + "-" + (futureYear + 10),
                classes: {},
            },
            rows: rowNumbers.map(rowOfYears.bind(this))
        };
        result.leftButton.classes["" + result.leftButton.value] = true;
        result.rightButton.classes["" + result.rightButton.value] = true;
        return result;
        function rowOfYears(rowNumber) {
            var currentMoment = moment();
            var cells = columnNumbers.map(function (columnNumber) {
                var yearMoment = moment(startDate).add((rowNumber * columnNumbers.length) + columnNumber, 'years');
                return {
                    display: yearMoment.format('YYYY'),
                    value: yearMoment.valueOf(),
                    classes: {
                        'dl-abdtp-active': activeValue === yearMoment.valueOf(),
                        'dl-abdtp-selected': selectedValue === yearMoment.valueOf(),
                        'dl-abdtp-now': yearMoment.isSame(currentMoment, 'year'),
                    }
                };
            });
            return { cells: cells };
        }
    };
    DlYearModelComponent.prototype.goDown = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment(fromMilliseconds).add(5, 'year').valueOf(), selectedMilliseconds);
    };
    DlYearModelComponent.prototype.goUp = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment(fromMilliseconds).subtract(5, 'year').valueOf(), selectedMilliseconds);
    };
    DlYearModelComponent.prototype.goLeft = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment(fromMilliseconds).subtract(1, 'year').valueOf(), selectedMilliseconds);
    };
    DlYearModelComponent.prototype.goRight = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment(fromMilliseconds).add(1, 'year').valueOf(), selectedMilliseconds);
    };
    DlYearModelComponent.prototype.pageDown = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment(fromMilliseconds).add(10, 'year').valueOf(), selectedMilliseconds);
    };
    DlYearModelComponent.prototype.pageUp = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment(fromMilliseconds).subtract(10, 'year').valueOf(), selectedMilliseconds);
    };
    DlYearModelComponent.prototype.goEnd = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(DlYearModelComponent.getStartOfDecade(fromMilliseconds)
            .add(9, 'years')
            .endOf('year')
            .valueOf(), selectedMilliseconds);
    };
    DlYearModelComponent.prototype.goHome = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(DlYearModelComponent.getStartOfDecade(fromMilliseconds)
            .startOf('year')
            .valueOf(), selectedMilliseconds);
    };
    return DlYearModelComponent;
}());
DlYearModelComponent.decorators = [
    { type: Component, args: [{
                providers: [
                    {
                        provide: DlYearModelComponent,
                        useClass: DlYearModelComponent,
                    },
                ],
            },] },
];
DlYearModelComponent.ctorParameters = function () { return []; };
var moment$1 = _moment;
var DlMonthModelComponent = /** @class */ (function () {
    function DlMonthModelComponent() {
    }
    DlMonthModelComponent.prototype.onChanges = function (changes) {
    };
    DlMonthModelComponent.prototype.getModel = function (milliseconds, selectedMilliseconds) {
        var startDate = moment$1(milliseconds).startOf('year');
        var rowNumbers = [0, 1, 2];
        var columnNumbers = [0, 1, 2, 3];
        var previousYear = moment$1(startDate).subtract(1, 'year');
        var nextYear = moment$1(startDate).add(1, 'year');
        var activeValue = moment$1(milliseconds).startOf('month').valueOf();
        var selectedValue = selectedMilliseconds === null || selectedMilliseconds === undefined
            ? selectedMilliseconds
            : moment$1(selectedMilliseconds).startOf('month').valueOf();
        var result = {
            viewName: 'month',
            viewLabel: startDate.format('YYYY'),
            activeDate: activeValue,
            leftButton: {
                value: previousYear.valueOf(),
                ariaLabel: "Go to " + previousYear.format('YYYY'),
                classes: {},
            },
            upButton: {
                value: startDate.valueOf(),
                ariaLabel: "Go to " + startDate.format('YYYY'),
                classes: {},
            },
            rightButton: {
                value: nextYear.valueOf(),
                ariaLabel: "Go to " + nextYear.format('YYYY'),
                classes: {},
            },
            rows: rowNumbers.map(rowOfMonths)
        };
        result.leftButton.classes["" + result.leftButton.value] = true;
        result.rightButton.classes["" + result.rightButton.value] = true;
        return result;
        function rowOfMonths(rowNumber) {
            var currentMoment = moment$1();
            var cells = columnNumbers.map(function (columnNumber) {
                var monthMoment = moment$1(startDate).add((rowNumber * columnNumbers.length) + columnNumber, 'months');
                return {
                    display: monthMoment.format('MMM'),
                    ariaLabel: monthMoment.format('MMM YYYY'),
                    value: monthMoment.valueOf(),
                    classes: {
                        'dl-abdtp-active': activeValue === monthMoment.valueOf(),
                        'dl-abdtp-selected': selectedValue === monthMoment.valueOf(),
                        'dl-abdtp-now': monthMoment.isSame(currentMoment, 'month'),
                    }
                };
            });
            return { cells: cells };
        }
    };
    DlMonthModelComponent.prototype.goDown = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$1(fromMilliseconds).add(4, 'month').valueOf(), selectedMilliseconds);
    };
    DlMonthModelComponent.prototype.goUp = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$1(fromMilliseconds).subtract(4, 'month').valueOf(), selectedMilliseconds);
    };
    DlMonthModelComponent.prototype.goLeft = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$1(fromMilliseconds).subtract(1, 'month').valueOf(), selectedMilliseconds);
    };
    DlMonthModelComponent.prototype.goRight = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$1(fromMilliseconds).add(1, 'month').valueOf(), selectedMilliseconds);
    };
    DlMonthModelComponent.prototype.pageDown = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$1(fromMilliseconds).add(12, 'months').valueOf(), selectedMilliseconds);
    };
    DlMonthModelComponent.prototype.pageUp = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$1(fromMilliseconds).subtract(12, 'months').valueOf(), selectedMilliseconds);
    };
    DlMonthModelComponent.prototype.goEnd = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$1(fromMilliseconds).endOf('year').valueOf(), selectedMilliseconds);
    };
    DlMonthModelComponent.prototype.goHome = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$1(fromMilliseconds).startOf('year').valueOf(), selectedMilliseconds);
    };
    return DlMonthModelComponent;
}());
DlMonthModelComponent.decorators = [
    { type: Component, args: [{
                providers: [
                    {
                        provide: DlMonthModelComponent,
                        useClass: DlMonthModelComponent,
                    },
                ],
            },] },
];
DlMonthModelComponent.ctorParameters = function () { return []; };
var moment$2 = _moment;
var DlDayModelComponent = /** @class */ (function () {
    function DlDayModelComponent() {
    }
    DlDayModelComponent.prototype.onChanges = function (changes) {
    };
    DlDayModelComponent.prototype.getModel = function (milliseconds, selectedMilliseconds) {
        var startOfMonth = moment$2(milliseconds).startOf('month');
        var endOfMonth = moment$2(milliseconds).endOf('month');
        var startOfView = moment$2(startOfMonth).subtract(Math.abs(startOfMonth.weekday()), 'days');
        var rowNumbers = [0, 1, 2, 3, 4, 5];
        var columnNumbers = [0, 1, 2, 3, 4, 5, 6];
        var previousMonth = moment$2(startOfMonth).subtract(1, 'month');
        var nextMonth = moment$2(startOfMonth).add(1, 'month');
        var activeValue = moment$2(milliseconds).startOf('day').valueOf();
        var selectedValue = selectedMilliseconds === null || selectedMilliseconds === undefined
            ? selectedMilliseconds
            : moment$2(selectedMilliseconds).startOf('day').valueOf();
        var result = {
            viewName: 'day',
            viewLabel: startOfMonth.format('MMM YYYY'),
            activeDate: activeValue,
            leftButton: {
                value: previousMonth.valueOf(),
                ariaLabel: "Go to " + previousMonth.format('MMM YYYY'),
                classes: {},
            },
            upButton: {
                value: startOfMonth.valueOf(),
                ariaLabel: "Go to month view",
                classes: {},
            },
            rightButton: {
                value: nextMonth.valueOf(),
                ariaLabel: "Go to " + nextMonth.format('MMM YYYY'),
                classes: {},
            },
            rowLabels: columnNumbers.map(function (column) { return moment$2().weekday(column).format('dd'); }),
            rows: rowNumbers.map(rowOfDays)
        };
        result.leftButton.classes["" + result.leftButton.value] = true;
        result.rightButton.classes["" + result.rightButton.value] = true;
        return result;
        function rowOfDays(rowNumber) {
            var currentMoment = moment$2();
            var cells = columnNumbers.map(function (columnNumber) {
                var dayMoment = moment$2(startOfView).add((rowNumber * columnNumbers.length) + columnNumber, 'days');
                return {
                    display: dayMoment.format('D'),
                    ariaLabel: dayMoment.format('ll'),
                    value: dayMoment.valueOf(),
                    classes: {
                        'dl-abdtp-active': activeValue === dayMoment.valueOf(),
                        'dl-abdtp-future': dayMoment.isAfter(endOfMonth),
                        'dl-abdtp-past': dayMoment.isBefore(startOfMonth),
                        'dl-abdtp-selected': selectedValue === dayMoment.valueOf(),
                        'dl-abdtp-now': dayMoment.isSame(currentMoment, 'day'),
                    }
                };
            });
            return { cells: cells };
        }
    };
    DlDayModelComponent.prototype.goDown = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$2(fromMilliseconds).add(7, 'days').valueOf(), selectedMilliseconds);
    };
    DlDayModelComponent.prototype.goUp = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$2(fromMilliseconds).subtract(7, 'days').valueOf(), selectedMilliseconds);
    };
    DlDayModelComponent.prototype.goLeft = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$2(fromMilliseconds).subtract(1, 'day').valueOf(), selectedMilliseconds);
    };
    DlDayModelComponent.prototype.goRight = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$2(fromMilliseconds).add(1, 'day').valueOf(), selectedMilliseconds);
    };
    DlDayModelComponent.prototype.pageDown = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$2(fromMilliseconds).add(1, 'month').valueOf(), selectedMilliseconds);
    };
    DlDayModelComponent.prototype.pageUp = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$2(fromMilliseconds).subtract(1, 'month').valueOf(), selectedMilliseconds);
    };
    DlDayModelComponent.prototype.goEnd = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$2(fromMilliseconds)
            .endOf('month').startOf('day').valueOf(), selectedMilliseconds);
    };
    DlDayModelComponent.prototype.goHome = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$2(fromMilliseconds).startOf('month').valueOf(), selectedMilliseconds);
    };
    return DlDayModelComponent;
}());
DlDayModelComponent.decorators = [
    { type: Component, args: [{
                providers: [
                    {
                        provide: DlDayModelComponent,
                        useClass: DlDayModelComponent,
                    },
                ],
            },] },
];
DlDayModelComponent.ctorParameters = function () { return []; };
var moment$3 = _moment;
var DlHourModelComponent = /** @class */ (function () {
    function DlHourModelComponent() {
    }
    DlHourModelComponent.prototype.onChanges = function (changes) {
    };
    DlHourModelComponent.prototype.getModel = function (milliseconds, selectedMilliseconds) {
        var startDate = moment$3(milliseconds).startOf('day');
        var rowNumbers = [0, 1, 2, 3, 4, 5];
        var columnNumbers = [0, 1, 2, 3];
        var previousDay = moment$3(startDate).subtract(1, 'day');
        var nextDay = moment$3(startDate).add(1, 'day');
        var activeValue = moment$3(milliseconds).startOf('hour').valueOf();
        var selectedValue = selectedMilliseconds === null || selectedMilliseconds === undefined
            ? selectedMilliseconds
            : moment$3(selectedMilliseconds).startOf('hour').valueOf();
        var result = {
            viewName: 'hour',
            viewLabel: startDate.format('ll'),
            activeDate: activeValue,
            leftButton: {
                value: previousDay.valueOf(),
                ariaLabel: "Go to " + previousDay.format('ll'),
                classes: {},
            },
            upButton: {
                value: startDate.valueOf(),
                ariaLabel: "Go to " + startDate.format('MMM YYYY'),
                classes: {},
            },
            rightButton: {
                value: nextDay.valueOf(),
                ariaLabel: "Go to " + nextDay.format('ll'),
                classes: {},
            },
            rows: rowNumbers.map(rowOfHours)
        };
        result.leftButton.classes["" + result.leftButton.value] = true;
        result.rightButton.classes["" + result.rightButton.value] = true;
        return result;
        function rowOfHours(rowNumber) {
            var currentMoment = moment$3();
            var cells = columnNumbers.map(function (columnNumber) {
                var hourMoment = moment$3(startDate).add((rowNumber * columnNumbers.length) + columnNumber, 'hours');
                return {
                    display: hourMoment.format('LT'),
                    ariaLabel: hourMoment.format('LLL'),
                    value: hourMoment.valueOf(),
                    classes: {
                        'dl-abdtp-active': activeValue === hourMoment.valueOf(),
                        'dl-abdtp-selected': selectedValue === hourMoment.valueOf(),
                        'dl-abdtp-now': hourMoment.isSame(currentMoment, 'hour'),
                    }
                };
            });
            return { cells: cells };
        }
    };
    DlHourModelComponent.prototype.goDown = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$3(fromMilliseconds).add(4, 'hour').valueOf(), selectedMilliseconds);
    };
    DlHourModelComponent.prototype.goUp = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$3(fromMilliseconds).subtract(4, 'hour').valueOf(), selectedMilliseconds);
    };
    DlHourModelComponent.prototype.goLeft = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$3(fromMilliseconds).subtract(1, 'hour').valueOf(), selectedMilliseconds);
    };
    DlHourModelComponent.prototype.goRight = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$3(fromMilliseconds).add(1, 'hour').valueOf(), selectedMilliseconds);
    };
    DlHourModelComponent.prototype.pageDown = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$3(fromMilliseconds).add(1, 'day').valueOf(), selectedMilliseconds);
    };
    DlHourModelComponent.prototype.pageUp = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$3(fromMilliseconds).subtract(1, 'day').valueOf(), selectedMilliseconds);
    };
    DlHourModelComponent.prototype.goEnd = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$3(fromMilliseconds)
            .endOf('day')
            .startOf('hour')
            .valueOf(), selectedMilliseconds);
    };
    DlHourModelComponent.prototype.goHome = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$3(fromMilliseconds).startOf('day').valueOf(), selectedMilliseconds);
    };
    return DlHourModelComponent;
}());
DlHourModelComponent.decorators = [
    { type: Component, args: [{
                providers: [
                    {
                        provide: DlHourModelComponent,
                        useClass: DlHourModelComponent,
                    },
                ],
            },] },
];
DlHourModelComponent.ctorParameters = function () { return []; };
var moment$4 = _moment;
var DlMinuteModelComponent = /** @class */ (function () {
    function DlMinuteModelComponent() {
        this.step = 5;
    }
    DlMinuteModelComponent.prototype.onChanges = function (changes) {
        var minuteStepChange = changes['minuteStep'];
        if (minuteStepChange
            && (minuteStepChange.previousValue !== minuteStepChange.currentValue)) {
            this.step = minuteStepChange.currentValue;
            if (this.step === null || this.step === undefined) {
                this.step = 5;
            }
        }
    };
    DlMinuteModelComponent.prototype.getModel = function (milliseconds, selectedMilliseconds) {
        var _this = this;
        var startDate = moment$4(milliseconds).startOf('hour');
        var currentMilliseconds = moment$4().valueOf();
        var minuteSteps = new Array(Math.ceil(60 / this.step)).fill(0).map(function (value, index) { return index * _this.step; });
        var minuteValues = minuteSteps.map(function (minutesToAdd) { return moment$4(startDate).add(minutesToAdd, 'minutes').valueOf(); });
        var activeValue = moment$4(minuteValues.filter(function (value) { return value <= milliseconds; }).pop()).valueOf();
        var nowValue = currentMilliseconds >= startDate.valueOf() && currentMilliseconds <= moment$4(startDate).endOf('hour').valueOf()
            ? moment$4(minuteValues.filter(function (value) { return value <= currentMilliseconds; }).pop()).valueOf()
            : null;
        var previousHour = moment$4(startDate).subtract(1, 'hour');
        var nextHour = moment$4(startDate).add(1, 'hour');
        var selectedValue = selectedMilliseconds === null || selectedMilliseconds === undefined
            ? selectedMilliseconds
            : moment$4(minuteValues.filter(function (value) { return value <= selectedMilliseconds; }).pop()).valueOf();
        var rows = new Array(Math.ceil(minuteSteps.length / 4))
            .fill(0)
            .map(function (value, index) { return index; })
            .map(function (value) {
            return { cells: minuteSteps.slice((value * 4), (value * 4) + 4).map(rowOfMinutes) };
        });
        var result = {
            viewName: 'minute',
            viewLabel: startDate.format('lll'),
            activeDate: activeValue,
            leftButton: {
                value: previousHour.valueOf(),
                ariaLabel: "Go to " + previousHour.format('lll'),
                classes: {},
            },
            upButton: {
                value: startDate.valueOf(),
                ariaLabel: "Go to " + startDate.format('ll'),
                classes: {},
            },
            rightButton: {
                value: nextHour.valueOf(),
                ariaLabel: "Go to " + nextHour.format('lll'),
                classes: {},
            },
            rows: rows
        };
        result.leftButton.classes["" + result.leftButton.value] = true;
        result.rightButton.classes["" + result.rightButton.value] = true;
        return result;
        function rowOfMinutes(stepMinutes) {
            var minuteMoment = moment$4(startDate).add(stepMinutes, 'minutes');
            return {
                display: minuteMoment.format('LT'),
                ariaLabel: minuteMoment.format('LLL'),
                value: minuteMoment.valueOf(),
                classes: {
                    'dl-abdtp-active': activeValue === minuteMoment.valueOf(),
                    'dl-abdtp-selected': selectedValue === minuteMoment.valueOf(),
                    'dl-abdtp-now': nowValue === minuteMoment.valueOf(),
                }
            };
        }
    };
    DlMinuteModelComponent.prototype.goDown = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$4(fromMilliseconds).add(this.step * 4, 'minutes').valueOf(), selectedMilliseconds);
    };
    DlMinuteModelComponent.prototype.goUp = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$4(fromMilliseconds).subtract(this.step * 4, 'minutes').valueOf(), selectedMilliseconds);
    };
    DlMinuteModelComponent.prototype.goLeft = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$4(fromMilliseconds).subtract(this.step, 'minutes').valueOf(), selectedMilliseconds);
    };
    DlMinuteModelComponent.prototype.goRight = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$4(fromMilliseconds).add(this.step, 'minutes').valueOf(), selectedMilliseconds);
    };
    DlMinuteModelComponent.prototype.pageDown = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$4(fromMilliseconds).add(1, 'hour').valueOf(), selectedMilliseconds);
    };
    DlMinuteModelComponent.prototype.pageUp = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$4(fromMilliseconds).subtract(1, 'hour').valueOf(), selectedMilliseconds);
    };
    DlMinuteModelComponent.prototype.goEnd = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$4(fromMilliseconds)
            .endOf('hour')
            .valueOf(), selectedMilliseconds);
    };
    DlMinuteModelComponent.prototype.goHome = function (fromMilliseconds, selectedMilliseconds) {
        return this.getModel(moment$4(fromMilliseconds).startOf('hour').valueOf(), selectedMilliseconds);
    };
    return DlMinuteModelComponent;
}());
DlMinuteModelComponent.decorators = [
    { type: Component, args: [{
                providers: [
                    {
                        provide: DlMinuteModelComponent,
                        useClass: DlMinuteModelComponent,
                    },
                ],
            },] },
];
DlMinuteModelComponent.ctorParameters = function () { return []; };
var DlDateTimePickerChange = /** @class */ (function () {
    function DlDateTimePickerChange(milliseconds) {
        this._value = milliseconds;
    }
    Object.defineProperty(DlDateTimePickerChange.prototype, "value", {
        get: function () {
            return this._value;
        },
        enumerable: true,
        configurable: true
    });
    return DlDateTimePickerChange;
}());
var moment$5 = _moment;
var keyCodeToModelProviderMethod = {
    33: 'pageUp',
    34: 'pageDown',
    35: 'goEnd',
    36: 'goHome',
    37: 'goLeft',
    38: 'goUp',
    39: 'goRight',
    40: 'goDown',
};
var VIEWS = [
    'minute',
    'hour',
    'day',
    'month',
    'year'
];
var DlDateTimePickerComponent = /** @class */ (function () {
    function DlDateTimePickerComponent(_elementRef, _ngZone, yearModelComponent, monthModelComponent, dayModelComponent, hourModelComponent, minuteModelComponent) {
        this._elementRef = _elementRef;
        this._ngZone = _ngZone;
        this.yearModelComponent = yearModelComponent;
        this.monthModelComponent = monthModelComponent;
        this.dayModelComponent = dayModelComponent;
        this.hourModelComponent = hourModelComponent;
        this.minuteModelComponent = minuteModelComponent;
        this.leftIconClass = [
            'oi',
            'oi-chevron-left'
        ];
        this.maxView = 'year';
        this.minuteStep = 5;
        this.minView = 'minute';
        this.rightIconClass = [
            'oi',
            'oi-chevron-right'
        ];
        this.startView = 'day';
        this.upIconClass = [
            'oi',
            'oi-chevron-top'
        ];
        this.change = new EventEmitter();
        this._changed = [];
        this._nextView = {
            'year': 'month',
            'month': 'day',
            'day': 'hour',
            'hour': 'minute'
        };
        this._previousView = {
            'minute': 'hour',
            'hour': 'day',
            'day': 'month',
            'month': 'year'
        };
        this._touched = [];
        this.selectFilter = function () { return true; };
        this._viewToModelProvider = {
            year: yearModelComponent,
            month: monthModelComponent,
            day: dayModelComponent,
            hour: hourModelComponent,
            minute: minuteModelComponent,
        };
    }
    DlDateTimePickerComponent.prototype.ngOnChanges = function (changes) {
        var _this = this;
        Object.keys(this._viewToModelProvider)
            .map(function (key) { return _this._viewToModelProvider[key]; })
            .forEach(function (provider) { return provider.onChanges(changes); });
        if (this._model) {
            this.model = this._viewToModelProvider[this._model.viewName].getModel(this._model.activeDate, this.value);
        }
    };
    DlDateTimePickerComponent.prototype.ngOnInit = function () {
        this.model = this._viewToModelProvider[this.getStartView()].getModel(this.getStartDate(), this.value);
    };
    DlDateTimePickerComponent.prototype._onDateClick = function (dateButton) {
        if (dateButton.classes['dl-abdtp-disabled']) {
            return;
        }
        var nextView = this._nextView[this._model.viewName];
        if ((this.minView || 'minute') === this._model.viewName) {
            this.value = dateButton.value;
            nextView = this.startView;
        }
        this.model = this._viewToModelProvider[nextView].getModel(dateButton.value, this.value);
        this.onTouch();
    };
    DlDateTimePickerComponent.prototype._onLeftClick = function () {
        this.model = this._viewToModelProvider[this._model.viewName].getModel(this._model.leftButton.value, this.value);
        this.onTouch();
    };
    DlDateTimePickerComponent.prototype._onUpClick = function () {
        this.model = this._viewToModelProvider[this._previousView[this._model.viewName]].getModel(this._model.upButton.value, this.value);
    };
    DlDateTimePickerComponent.prototype._onRightClick = function () {
        this.model = this._viewToModelProvider[this._model.viewName].getModel(this._model.rightButton.value, this.value);
        this.onTouch();
    };
    DlDateTimePickerComponent.prototype._handleKeyDown = function ($event) {
        var functionName = keyCodeToModelProviderMethod[$event.keyCode];
        if (functionName) {
            var modelProvider = this._viewToModelProvider[this._model.viewName];
            this.model = modelProvider[functionName](this._model.activeDate, this.value);
            this.focusActiveCell();
            event.preventDefault();
        }
    };
    DlDateTimePickerComponent.prototype.applySelectFilter = function (model) {
        var _this = this;
        if (this.selectFilter) {
            model.rows = model.rows.map(function (row) {
                row.cells.map(function (dateButton) {
                    var disabled = !_this.selectFilter(dateButton, model.viewName);
                    dateButton.classes['dl-abdtp-disabled'] = disabled;
                    if (disabled) {
                        dateButton.classes['aria-disabled'] = true;
                    }
                    return dateButton;
                });
                return row;
            });
        }
        return model;
    };
    DlDateTimePickerComponent.prototype.focusActiveCell = function () {
        var _this = this;
        this._ngZone.runOutsideAngular(function () {
            _this._ngZone.onStable.asObservable().pipe(take(1)).subscribe(function () {
                _this._elementRef.nativeElement.querySelector('.dl-abdtp-active').focus();
            });
        });
    };
    DlDateTimePickerComponent.prototype.getStartDate = function () {
        if (hasValue(this._value)) {
            return this._value;
        }
        if (hasValue(this.startDate)) {
            return this.startDate;
        }
        return moment$5().valueOf();
    };
    DlDateTimePickerComponent.prototype.getStartView = function () {
        var startIndex = Math.max(VIEWS.indexOf(this.minView || 'minute'), VIEWS.indexOf(this.startView || 'day'));
        return VIEWS[startIndex];
    };
    Object.defineProperty(DlDateTimePickerComponent.prototype, "model", {
        set: function (model) {
            this._model = this.applySelectFilter(model);
        },
        enumerable: true,
        configurable: true
    });
    DlDateTimePickerComponent.prototype.onTouch = function () {
        this._touched.forEach(function (onTouch) { return onTouch(); });
    };
    DlDateTimePickerComponent.prototype.registerOnChange = function (fn) {
        this._changed.push(fn);
    };
    DlDateTimePickerComponent.prototype.registerOnTouched = function (fn) {
        this._touched.push(fn);
    };
    Object.defineProperty(DlDateTimePickerComponent.prototype, "value", {
        get: function () {
            return this._value;
        },
        set: function (value) {
            if (this._value !== value) {
                this._value = value;
                this.model = this._viewToModelProvider[this._model.viewName].getModel(this.getStartDate(), this.value);
                this._changed.forEach(function (f) { return f(value); });
                this.change.emit(new DlDateTimePickerChange(value));
            }
        },
        enumerable: true,
        configurable: true
    });
    DlDateTimePickerComponent.prototype.writeValue = function (value) {
        this.value = value;
    };
    return DlDateTimePickerComponent;
}());
DlDateTimePickerComponent.decorators = [
    { type: Component, args: [{
                changeDetection: ChangeDetectionStrategy.OnPush,
                preserveWhitespaces: false,
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: DlDateTimePickerComponent,
                        multi: true
                    }
                ],
                selector: 'dl-date-time-picker',
                styles: [":host{\n  -webkit-user-select:none;\n     -moz-user-select:none;\n      -ms-user-select:none;\n          user-select:none; }\n.dl-abdtp-col-label,\n.dl-abdtp-view-label{\n  font-weight:bold; }\n.dl-abdtp-view-label,\n.dl-abdtp-left-button,\n.dl-abdtp-right-button,\n.dl-abdtp-date-button{\n  padding:5px;\n  border-radius:999px;\n  cursor:pointer;\n  color:rgba(0, 0, 0, 0.87);\n  outline:0; }\n.dl-abdtp-left-button,\n.dl-abdtp-up-button,\n.dl-abdtp-right-button,\n.dl-abdtp-date-button{\n  border-width:0; }\n.dl-abdtp-active:focus,\n.dl-abdtp-date-button:focus,\n.dl-abdtp-date-button:hover,\n.dl-abdtp-left-button:focus,\n.dl-abdtp-left-button:hover,\n.dl-abdtp-right-button:focus,\n.dl-abdtp-right-button:hover,\n.dl-abdtp-up-button:focus,\n.dl-abdtp-up-button:hover,\n.dl-abdtp-view-label:focus{\n  background:rgba(0, 0, 0, 0.04); }\n.dl-abdtp-past,\n.dl-abdtp-future{\n  color:rgba(0, 0, 0, 0.04); }\n.dl-abdtp-now,\n.dl-abdtp-now:hover,\n.dl-abdtp-now.disabled,\n.dl-abdtp-now.disabled:hover{\n  border-width:1px;\n  border-style:solid;\n  border-radius:999px;\n  border-color:rgba(0, 0, 0, 0.25); }\n.dl-abdtp-selected{\n  color:#ffffff;\n  background:rgba(0, 82, 204, 0.75); }\n.dl-abdtp-selected:focus,\n.dl-abdtp-selected:hover{\n  background:#0052cc; }\n.dl-abdtp-disabled{\n  cursor:default;\n  color:rgba(0, 0, 0, 0.25); }\n"],
                template: "<div class=\"text-center dl-abdtp-{{_model.viewName}}-view \">\n  <div class=\"row align-items-center no-gutters\">\n    <button class=\"col dl-abdtp-left-button align-items-center\"\n            type=\"button\"\n            [attr.aria-label]=\"_model.leftButton.ariaLabel\"\n            [attr.title]=\"_model.leftButton.ariaLabel\"\n            [ngClass]=\"_model.leftButton.classes\"\n            (click)=\"_onLeftClick()\">\n      <span class=\"left-icon\" [ngClass]=\"leftIconClass\"></span>\n    </button>\n    <div *ngIf=\"_model.viewName === (this.maxView || 'year'); then maxViewLabel else defaultViewLabel;\"></div>\n    <button class=\"col dl-abdtp-right-button\"\n            type=\"button\"\n            [attr.aria-label]=\"_model.rightButton.ariaLabel\"\n            [attr.title]=\"_model.rightButton.ariaLabel\"\n            (click)=\"_onRightClick()\"\n            [ngClass]=\"_model.rightButton.classes\">\n      <span class=\"right-icon\" [ngClass]=\"rightIconClass\"></span>\n    </button>\n  </div>\n  <div (keydown)=\"_handleKeyDown($event)\">\n    <div *ngIf=\"_model.rowLabels?.length\" class=\"row no-gutters\">\n      <div *ngFor=\"let label of _model.rowLabels\" class=\"col align-items-center no-gutters dl-abdtp-col-label\">{{label}}</div>\n    </div>\n    <div *ngFor=\"let row of _model.rows\" class=\"row align-items-center no-gutters\">\n      <div *ngFor=\"let cell of row.cells\"\n           role=\"gridcell\"\n           class=\"col dl-abdtp-date-button dl-abdtp-{{_model.viewName}} {{cell.value}}\"\n           [ngClass]=\"cell.classes\"\n           [attr.aria-label]=\"cell.ariaLabel\"\n           [attr.aria-disabled]=\"cell.classes['dl-abdtp-disabled']\"\n           [attr.tabindex]=\"cell.classes['dl-abdtp-active'] ? 0 : -1\"\n           (click)=\"_onDateClick(cell)\"\n           (keydown.space)=\"_onDateClick(cell)\"\n           (keydown.enter)=\"_onDateClick(cell)\"\n      >{{cell.display}}</div>\n    </div>\n  </div>\n</div>\n<ng-template #maxViewLabel>\n  <div class=\"col-10 dl-abdtp-view-label\">{{_model.viewLabel}}</div>\n</ng-template>\n<ng-template #defaultViewLabel>\n  <button class=\"col-10 dl-abdtp-view-label dl-abdtp-up-button\"\n          type=\"button\"\n          [attr.aria-label]=\"_model.upButton.ariaLabel\"\n          [attr.title]=\"_model.upButton.ariaLabel\"\n          (click)=\"_onUpClick()\"\n          [ngClass]=\"_model.upButton.classes\"\n  >{{_model.viewLabel}}&nbsp;<span class=\"up-icon\" [ngClass]=\"upIconClass\"></span>\n  </button>\n</ng-template>\n",
            },] },
];
DlDateTimePickerComponent.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: NgZone, },
    { type: DlYearModelComponent, },
    { type: DlMonthModelComponent, },
    { type: DlDayModelComponent, },
    { type: DlHourModelComponent, },
    { type: DlMinuteModelComponent, },
]; };
DlDateTimePickerComponent.propDecorators = {
    "leftIconClass": [{ type: Input },],
    "maxView": [{ type: Input },],
    "minuteStep": [{ type: Input },],
    "minView": [{ type: Input },],
    "rightIconClass": [{ type: Input },],
    "startDate": [{ type: Input },],
    "startView": [{ type: Input },],
    "upIconClass": [{ type: Input },],
    "change": [{ type: Output },],
    "selectFilter": [{ type: Input },],
};
function hasValue(value) {
    return (typeof value !== 'undefined') && (value !== null);
}
var DlDateTimePickerModule = /** @class */ (function () {
    function DlDateTimePickerModule() {
    }
    return DlDateTimePickerModule;
}());
DlDateTimePickerModule.decorators = [
    { type: NgModule, args: [{
                declarations: [DlDateTimePickerComponent],
                imports: [CommonModule],
                exports: [DlDateTimePickerComponent],
                providers: [
                    DlYearModelComponent,
                    DlMonthModelComponent,
                    DlDayModelComponent,
                    DlHourModelComponent,
                    DlMinuteModelComponent
                ],
            },] },
];
DlDateTimePickerModule.ctorParameters = function () { return []; };

export { DlDateTimePickerModule, DlDateTimePickerComponent, DlDateTimePickerChange, DlDayModelComponent, DlHourModelComponent, DlMinuteModelComponent, DlMonthModelComponent, DlYearModelComponent };
//# sourceMappingURL=angular-bootstrap-datetimepicker.js.map
